@@ -6,7 +6,7 @@
 use std::error::Error;
 use std::fmt::Write as _; // import without risk of name clashing
 use std::fs::{self, File};
-use std::io::{self, Read, Write, stdout};
+use std::io::{self, IsTerminal, Read, Write, stdout};
 use std::path::{Path, PathBuf};
 use std::sync::{Arc, Mutex};
 use std::{env, fmt};
@@ -29,6 +29,7 @@ use srgn::scoping::Scoper;
 use srgn::scoping::langs::LanguageScoper;
 use srgn::scoping::literal::{Literal, LiteralError};
 use srgn::scoping::regex::{Regex, RegexError};
+use srgn::scoping::scope::Scope;
 use srgn::scoping::view::ScopedViewBuilder;
 use tree_sitter::QueryError as TSQueryError;
 
@@ -48,7 +49,7 @@ fn main() -> Result<()> {
         .format_timestamp_micros() // High precision is nice for benchmarks
         .init();
 
-    info!("Launching app with args: {:?}", args);
+    info!("Launching app with args: {args:?}");
 
     let cli::Args {
         scope,
@@ -111,12 +112,29 @@ fn main() -> Result<()> {
         actions
     };
 
-    let is_readable_stdin = grep_cli::is_readable_stdin();
-    info!("Detected stdin as readable: {is_readable_stdin}.");
+    let is_stdout_tty = match options.stdout_detection {
+        cli::StdoutDetection::Auto => {
+            debug!("Detecting if stdout is a TTY");
+            stdout().is_terminal()
+        }
+        cli::StdoutDetection::ForceTTY => true,
+        cli::StdoutDetection::ForcePipe => false,
+    };
+    info!("Treating stdout as tty: {is_stdout_tty}.");
+
+    let is_stdin_readable = match options.stdin_detection {
+        cli::StdinDetection::Auto => {
+            debug!("Detecting if stdin is readable");
+            grep_cli::is_readable_stdin()
+        }
+        cli::StdinDetection::ForceReadable => true,
+        cli::StdinDetection::ForceUnreadable => false,
+    };
+    info!("Treating stdin as readable: {is_stdin_readable}.");
 
     // See where we're reading from
     let input = match (
-        options.stdin_override_to.unwrap_or(is_readable_stdin),
+        is_stdin_readable,
         options.glob.clone(),
         &language_scopers,
     ) {
@@ -173,7 +191,12 @@ fn main() -> Result<()> {
         } else {
             Style::red_bold() // "Found!", like ripgrep
         };
-        actions.push(Box::new(style));
+
+        if is_stdout_tty {
+            // For human consumption, we style - otherwise, none needed (in fact, color
+            // codes would mess with column positions, so omit).
+            actions.push(Box::new(style));
+        }
 
         options.only_matching = true;
         options.line_numbers = true;
@@ -208,6 +231,7 @@ fn main() -> Result<()> {
                 general_scoper.as_ref(),
                 &language_scopers,
                 &pipeline,
+                is_stdout_tty,
             )?;
         }
         (Input::WalkOn(validator), false) => {
@@ -224,6 +248,7 @@ fn main() -> Result<()> {
                     || std::thread::available_parallelism().map_or(1, std::num::NonZero::get),
                     std::num::NonZero::get,
                 ),
+                is_stdout_tty,
             )?;
         }
         (Input::WalkOn(validator), true) => {
@@ -236,6 +261,7 @@ fn main() -> Result<()> {
                 &language_scopers,
                 &pipeline,
                 search_mode,
+                is_stdout_tty,
             )?;
         }
     }
@@ -286,6 +312,7 @@ fn handle_actions_on_stdin(
     general_scoper: &dyn Scoper,
     language_scopers: &[Box<dyn LanguageScoper>],
     pipeline: Pipeline<'_>,
+    is_stdout_tty: bool,
 ) -> Result<(), ProgramError> {
     info!("Will use stdin to stdout.");
     let mut source = String::new();
@@ -300,6 +327,8 @@ fn handle_actions_on_stdin(
         general_scoper,
         language_scopers,
         pipeline,
+        None, // No filename for stdin
+        is_stdout_tty,
     )?;
 
     stdout().lock().write_all(destination.as_bytes())?;
@@ -315,6 +344,7 @@ fn handle_actions_on_stdin(
 ///
 /// [ripgrep]:
 ///     https://github.com/BurntSushi/ripgrep/blob/71d71d2d98964653cdfcfa315802f518664759d7/GUIDE.md#L1016-L1017
+#[expect(clippy::too_many_arguments)] // Yes :-(
 fn handle_actions_on_many_files_sorted(
     global_options: &cli::GlobalOptions,
     standalone_action: StandaloneAction,
@@ -323,6 +353,7 @@ fn handle_actions_on_many_files_sorted(
     language_scopers: &[Box<dyn LanguageScoper>],
     pipeline: Pipeline<'_>,
     search_mode: bool,
+    is_stdout_tty: bool,
 ) -> Result<(), ProgramError> {
     let root = env::current_dir()?;
     info!(
@@ -351,6 +382,7 @@ fn handle_actions_on_many_files_sorted(
                     language_scopers,
                     pipeline,
                     search_mode,
+                    is_stdout_tty,
                 );
 
                 n_files_seen += match res {
@@ -410,17 +442,17 @@ fn handle_actions_on_many_files_sorted(
             }
             Err(e) => {
                 if search_mode {
-                    error!("Error walking: {}", e);
+                    error!("Error walking: {e}");
                 } else {
-                    error!("Aborting walk due to: {}", e);
+                    error!("Aborting walk due to: {e}");
                     return Err(e.into());
                 }
             }
         }
     }
 
-    info!("Saw {} items", n_files_seen);
-    info!("Processed {} files", n_files_processed);
+    info!("Saw {n_files_seen} items");
+    info!("Processed {n_files_processed} files");
 
     if n_files_seen == 0 && global_options.fail_no_files {
         Err(ProgramError::NoFilesFound)
@@ -443,6 +475,7 @@ fn handle_actions_on_many_files_threaded(
     pipeline: Pipeline<'_>,
     search_mode: bool,
     n_threads: usize,
+    is_stdout_tty: bool,
 ) -> Result<(), ProgramError> {
     let root = env::current_dir()?;
     info!(
@@ -477,6 +510,7 @@ fn handle_actions_on_many_files_threaded(
                         language_scopers,
                         pipeline,
                         search_mode,
+                        is_stdout_tty,
                     );
 
                     match res {
@@ -544,10 +578,10 @@ fn handle_actions_on_many_files_threaded(
                 }
                 Err(e) => {
                     if search_mode {
-                        error!("Error walking: {}", e);
+                        error!("Error walking: {e}");
                         WalkState::Continue
                     } else {
-                        error!("Aborting walk due to: {}", e);
+                        error!("Aborting walk due to: {e}");
                         *err.lock().unwrap() = Some(e.into());
                         WalkState::Quit
                     }
@@ -561,9 +595,9 @@ fn handle_actions_on_many_files_threaded(
     }
 
     let n_files_seen = *n_files_seen.lock().unwrap();
-    info!("Saw {} items", n_files_seen);
+    info!("Saw {n_files_seen} items");
     let n_files_processed = *n_files_processed.lock().unwrap();
-    info!("Processed {} files", n_files_processed);
+    info!("Processed {n_files_processed} files");
 
     if n_files_seen == 0 && global_options.fail_no_files {
         Err(ProgramError::NoFilesFound)
@@ -585,20 +619,21 @@ fn process_path(
     language_scopers: &[Box<dyn LanguageScoper>],
     pipeline: Pipeline<'_>,
     search_mode: bool,
+    is_stdout_tty: bool,
 ) -> std::result::Result<(), PathProcessingError> {
     if !path.is_file() {
-        trace!("Skipping path (not a file): {:?}", path);
+        trace!("Skipping path (not a file): {}", path.display());
         return Err(PathProcessingError::NotAFile);
     }
 
     let path = diff_paths(path, root).expect("started walk at root, so relative to root works");
 
     if !validator(&path) {
-        trace!("Skipping path (invalid): {:?}", path);
+        trace!("Skipping path (invalid): {}", path.display());
         return Err(PathProcessingError::InvalidFile);
     }
 
-    debug!("Processing path: {:?}", path);
+    debug!("Processing path: {}", path.display());
 
     let (new_contents, filesize, changed) = {
         let mut file = File::open(&path)?;
@@ -618,6 +653,8 @@ fn process_path(
             general_scoper,
             language_scopers,
             pipeline,
+            Some(&path),
+            is_stdout_tty,
         )?;
 
         (destination, filesize, changed)
@@ -628,12 +665,19 @@ fn process_path(
 
     if search_mode {
         if !new_contents.is_empty() {
-            writeln!(
-                stdout,
-                "{}\n{}",
-                path.display().to_string().magenta(),
-                &new_contents
-            )?;
+            if is_stdout_tty {
+                // TTY format: colored filename with empty line separator
+                writeln!(
+                    stdout,
+                    "{}\n{}",
+                    path.display().to_string().magenta(),
+                    &new_contents
+                )?;
+            } else {
+                // Machine-parseable format: the content should already be formatted
+                // with filename:line_number: prefix by the apply function
+                write!(stdout, "{}", &new_contents)?;
+            }
         }
     } else {
         if filesize > 0 && new_contents.is_empty() {
@@ -642,15 +686,11 @@ fn process_path(
                 path.display(),
                 filesize
             );
-            return Err(io::Error::new(
-                io::ErrorKind::Other,
-                "attempt to wipe non-empty file (failsafe guard)",
-            )
-            .into());
+            return Err(io::Error::other("attempt to wipe non-empty file (failsafe guard)").into());
         }
 
         if changed {
-            debug!("Got new file contents, writing to file: {:?}", path);
+            debug!("Got new file contents, writing to file: {}", path.display());
             assert!(
                 !global_options.dry_run,
                 // Dry run leverages search mode, so should never get here. Assert for
@@ -668,7 +708,7 @@ fn process_path(
             );
         }
 
-        debug!("Done processing file: {:?}", path);
+        debug!("Done processing file: {}", path.display());
     }
 
     Ok(())
@@ -679,6 +719,7 @@ fn process_path(
 ///
 /// TODO: The way this interacts with [`process_path`] etc. is just **awful** spaghetti
 /// of the most imperative, procedural kind. Refactor needed.
+#[expect(clippy::too_many_arguments)] // Yes :-(
 fn apply(
     global_options: &cli::GlobalOptions,
     standalone_action: StandaloneAction,
@@ -689,6 +730,8 @@ fn apply(
     general_scoper: &dyn Scoper,
     language_scopers: &[Box<dyn LanguageScoper>],
     pipeline: Pipeline<'_>,
+    file_path: Option<&Path>,
+    is_stdout_tty: bool,
 ) -> std::result::Result<bool, ApplicationError> {
     debug!("Building view.");
     let mut builder = ScopedViewBuilder::new(source);
@@ -737,14 +780,64 @@ fn apply(
         for (i, lines) in line_based_views.into_iter().parallel_zip().enumerate() {
             let i = i + 1;
             for line in lines {
-                if !global_options.only_matching || line.has_any_in_scope() {
-                    if global_options.line_numbers {
-                        write!(destination, "{}:", i.to_string().green())
+                if global_options.only_matching && !line.has_any_in_scope() {
+                    continue;
+                }
+
+                let sep = ":";
+
+                if !is_stdout_tty {
+                    // Machine-parseable format
+                    if let Some(path) = file_path {
+                        write!(destination, "{}{sep}", path.display())
+                            .expect("infallible on String (are we OOM?)");
+                    } else {
+                        write!(destination, "(stdin){sep}")
                             .expect("infallible on String (are we OOM?)");
                     }
-
-                    destination.push_str(&line.to_string());
                 }
+
+                if global_options.line_numbers {
+                    write!(
+                        destination,
+                        "{}{sep}",
+                        if is_stdout_tty {
+                            i.to_string().green().to_string()
+                        } else {
+                            i.to_string()
+                        }
+                    )
+                    .expect("infallible on String (are we OOM?)");
+                }
+
+                if !is_stdout_tty {
+                    // This is for programmatic use: column positions are 0-indexed.
+                    let mut col = 0;
+
+                    let mut ranges = Vec::new();
+
+                    // NB: this manual column position computation should not live at
+                    // this layer; ideally a `Scope` knows how to serialize itself,
+                    // which would generalize to CLI output, JSON, ...
+                    for scope in &line.scopes().0 {
+                        let s: &str = scope.into();
+
+                        // This is for programmatic use: ranges are half-open, [from,
+                        // to).
+                        let end = col + s.len();
+
+                        if let Scope::In(..) = scope.0 {
+                            ranges.push(format!("{col}-{end}"));
+                        }
+
+                        col = end;
+                    }
+
+                    write!(destination, "{ranges}{sep}", ranges = ranges.join(";"))
+                        .expect("infallible on String (are we OOM?)");
+                }
+
+                destination.push_str(&line.to_string());
             }
         }
     } else {
@@ -1100,6 +1193,28 @@ mod cli {
         );
     }
 
+    /// Controls for stdin readability detection.
+    #[derive(Debug, Clone, ValueEnum)]
+    pub enum StdinDetection {
+        /// Automatically detect if stdin is readable.
+        Auto,
+        /// Act as if stdin is readable.
+        ForceReadable,
+        /// Act as if stdin is not readable.
+        ForceUnreadable,
+    }
+
+    /// Controls for stdout detection.
+    #[derive(Debug, Clone, ValueEnum)]
+    pub enum StdoutDetection {
+        /// Automatically detect if stdout is a TTY and act accordingly.
+        Auto,
+        /// Act as if stdout is a TTY.
+        ForceTTY,
+        /// Act as if stdout is not a TTY, e.g. a pipe, redirect.
+        ForcePipe,
+    }
+
     #[derive(Parser, Debug)]
     #[group(required = false, multiple = true)]
     #[command(next_help_heading = "Options (global)")]
@@ -1196,13 +1311,22 @@ mod cli {
         /// Sorted processing disables parallel processing.
         #[arg(long, verbatim_doc_comment)]
         pub sorted: bool,
-        /// Override detection heuristics for stdin readability, and force to value.
-        ///
-        /// `true` will always attempt to read from stdin. `false` will never read from
-        /// stdin, even if provided.
-        #[arg(long, hide(true), verbatim_doc_comment)]
-        // Hidden: internal use for testing, where some forceful overriding is required.
-        pub stdin_override_to: Option<bool>,
+        /// Control heuristics for stdin readability detection, and force to value.
+        #[arg(
+            long,
+            value_enum,
+            default_value_t=StdinDetection::Auto,
+            verbatim_doc_comment
+        )]
+        pub stdin_detection: StdinDetection,
+        /// Control heuristics for stdout detection, and potentially force to value.
+        #[arg(
+            long,
+            value_enum,
+            default_value_t=StdoutDetection::Auto,
+            verbatim_doc_comment
+        )]
+        pub stdout_detection: StdoutDetection,
         /// Number of threads to run processing on, when working with files.
         ///
         /// If not specified, will default to available parallelism. Set to 1 for
@@ -1239,7 +1363,7 @@ mod cli {
         /// compatibility with `tr`.
         ///
         /// If given, will run before any other action.
-        #[arg(value_name = "REPLACEMENT", env, verbatim_doc_comment)]
+        #[arg(value_name = "REPLACEMENT", env, verbatim_doc_comment, last = true)]
         pub replace: Option<String>,
         /// Uppercase anything in scope.
         #[arg(short, long, env, verbatim_doc_comment)]
