@@ -4,6 +4,7 @@
 #[cfg(test)]
 #[cfg(feature = "all")]
 mod tests {
+    use std::collections::VecDeque;
     use std::path::{Path, PathBuf};
 
     use anyhow::Context;
@@ -45,6 +46,7 @@ mod tests {
         false,
         &[
             "A",
+            "--",
             "B",
         ],
         Some(r"A;  B 😫"),
@@ -54,6 +56,7 @@ mod tests {
         false,
         &[
             "A",
+            "--",
             "B",
         ],
         None,
@@ -63,6 +66,7 @@ mod tests {
         false,
         &[
             r"\W",
+            "--",
             "B",
         ],
         Some(r"A;  B 😫"),
@@ -156,6 +160,7 @@ Heizoelrueckstossabdaempfung.
             "--go",
             "comments",
             "[fF]izz",
+            "--",
             "🤡",
         ],
         Some(include_str!("langs/go/fizzbuzz.go")),
@@ -233,6 +238,7 @@ Heizoelrueckstossabdaempfung.
         false,
         &[
             "A",
+            "--",
             "X",
         ],
         Some("A\nB"),
@@ -243,6 +249,7 @@ Heizoelrueckstossabdaempfung.
         &[
             "--only-matching",
             "A",
+            "--",
             "X",
         ],
         Some("A\nB"),
@@ -253,6 +260,7 @@ Heizoelrueckstossabdaempfung.
         &[
             "--line-numbers",
             "A",
+            "--",
             "X",
         ],
         Some("A\nB"),
@@ -264,6 +272,7 @@ Heizoelrueckstossabdaempfung.
             "--only-matching",
             "--line-numbers",
             "A",
+            "--",
             "X",
         ],
         Some("A\nB"),
@@ -298,20 +307,24 @@ Heizoelrueckstossabdaempfung.
         // (inside `src/main.rs` etc.).
         let mut cmd = get_cmd();
 
-        let args: Vec<String> = args.iter().map(|&s| s.to_owned()).collect();
+        let mut args: VecDeque<String> = args.iter().map(|&s| s.to_owned()).collect();
+
+        // Be deterministic for testing purposes. We have to push this awkwardly, to not
+        // override any potential positional arguments at the end.
+        args.push_front("1".into());
+        args.push_front("--threads".into());
 
-        cmd.args(args.clone());
-        cmd.args(["--threads", "1"]); // Be deterministic
         if let Some(stdin) = stdin {
             cmd.write_stdin(stdin);
         } else {
-            cmd.args(
-                // Override; `Command` is detected as providing stdin but we're working on
-                // files here.
-                ["--stdin-override-to", "false"],
-            );
+            // Override; `Command` is detected as providing stdin but we're working on
+            // files here.
+            args.push_front("force-unreadable".into());
+            args.push_front("--stdin-detection".into());
         }
 
+        cmd.args(args.clone());
+
         let output = cmd.output().expect("failed to execute process");
 
         let exit_code = output
@@ -332,7 +345,7 @@ Heizoelrueckstossabdaempfung.
             insta::assert_yaml_snapshot!(
                 snapshot_name,
                 CommandSnap {
-                    args,
+                    args: args.into(),
                     stdin: stdin.map(|s| s.split_inclusive('\n').map(ToOwned::to_owned).collect_vec()),
                     stdout: stdout.split_inclusive('\n').map(ToOwned::to_owned).collect_vec(),
                     exit_code,
@@ -351,6 +364,7 @@ Heizoelrueckstossabdaempfung.
             "--glob",
             "**/*.py",
             "foo",
+            "--",
             "baz"
         ],
         false,
@@ -364,6 +378,7 @@ Heizoelrueckstossabdaempfung.
             "--python",
             "function-names",
             "foo",
+            "--",
             "baz"
         ],
         false,
@@ -379,6 +394,7 @@ Heizoelrueckstossabdaempfung.
             "--glob", // Will override language scoper
             "subdir/**/*.py",
             "foo",
+            "--",
             "baz"
         ],
         false,
@@ -393,6 +409,7 @@ Heizoelrueckstossabdaempfung.
             "--glob", // Will override language scoper
             "subdir/**/*.py",
             "foo",
+            "--",
             "baz"
         ],
         // NOT `--sorted`, so not deterministic; use to test that directories are
@@ -409,6 +426,7 @@ Heizoelrueckstossabdaempfung.
             "--glob",
             "**/*",
             "0a1a09c8-2995-4ac5-9d60-01a0f02920e8",
+            "--",
             "gone"
         ],
         false,
@@ -421,6 +439,7 @@ Heizoelrueckstossabdaempfung.
             "--glob",
             "**/*",
             "0a1a09c8-2995-4ac5-9d60-01a0f02920e8",
+            "--",
             "gone"
         ],
         // NOT `--sorted`, so not deterministic; use to test that directories are
@@ -458,12 +477,12 @@ Heizoelrueckstossabdaempfung.
         cmd.args(
             // Override; `Command` is detected as providing stdin but we're working on
             // files here.
-            ["--stdin-override-to", "false"],
+            ["--stdin-detection", "force-unreadable"],
         );
-        cmd.args(&args);
         if dry_run {
             cmd.arg("--dry-run");
         }
+        cmd.args(&args);
 
         // Act
         let output = cmd.output().expect("failed to execute binary under test");
@@ -786,7 +805,7 @@ Heizoelrueckstossabdaempfung.
             cmd.args(
                 // Override; `Command` is detected as providing stdin but we're working on
                 // files here.
-                ["--stdin-override-to", "false"],
+                ["--stdin-detection", "force-unreadable"],
             );
         }
         cmd.args(&args);
@@ -959,10 +978,8 @@ Heizoelrueckstossabdaempfung.
                     .with_context(|| format!("Failure reading right file: {candidate:?}"))?;
 
                 if left_contents != right_contents {
-                    return Err(std::io::Error::new(
-                        std::io::ErrorKind::Other,
-                        format!(
-                            r"File contents differ:
+                    return Err(std::io::Error::other(format!(
+                        r"File contents differ:
 left path: {:?}
 right path: {:?}
 ---------
@@ -973,12 +990,11 @@ right contents:
 {}
 ---------
 ",
-                            left.path(),
-                            candidate,
-                            String::from_utf8_lossy(&left_contents).escape_debug(),
-                            String::from_utf8_lossy(&right_contents).escape_debug()
-                        ),
-                    )
+                        left.path(),
+                        candidate,
+                        String::from_utf8_lossy(&left_contents).escape_debug(),
+                        String::from_utf8_lossy(&right_contents).escape_debug()
+                    ))
                     .into());
                 }
             } else if metadata.is_dir() {
