@@ -243,7 +243,7 @@ This is a raw string with no special treatment for \n
 }
 
 #[cfg(feature = "some-feature")]
-#[allow(clippy::single_match_else)]
+#[expect(clippy::single_match_else)]
 #[cfg(doc)]
 fn function_with_attributes() {}
 
