@@ -170,9 +170,10 @@ mod tests {
         /// For that scenario, we need a hacky method to disable stdin for good.
         fn force_ignore_stdin(&mut self) {
             match self {
-                Self::Self_(inv) => inv
-                    .opts
-                    .push(Opt::Long("stdin-override-to".into(), "false".into())),
+                Self::Self_(inv) => inv.opts.push(Opt::Long(
+                    "stdin-detection".into(),
+                    "force-unreadable".into(),
+                )),
                 _ => panic!("Forcing stdin ignore only applicable to `self` program"),
             }
         }
@@ -191,10 +192,22 @@ mod tests {
                 Program::Self_(inv) => {
                     let mut cmd = Self::cargo_bin(name).expect("Should be able to find binary");
 
+                    // Push some test-wide flags which just help with test setup
+                    // (determinism, test environment control, ...). Push these first to
+                    // not interfere with any potential later positional arguments. NB:
+                    // *this hard-codes some flags*.
+                    cmd.args([
+                        // We're testing and need determinism.
+                        "--sorted",
+                    ]);
+
                     for flag in &inv.flags {
                         cmd.arg(flag.to_string());
                     }
 
+                    let stdout_detection_option = "stdout-detection";
+                    let mut stdout_detection_was_overridden = false;
+
                     for opt in inv.opts {
                         match opt {
                             Opt::Short(name, value) => {
@@ -204,11 +217,22 @@ mod tests {
                                 cmd.args([format!("-{name}"), value]);
                             }
                             Opt::Long(name, value) => {
+                                stdout_detection_was_overridden |= name == stdout_detection_option;
                                 cmd.args([format!("--{name}"), value]);
                             }
                         }
                     }
 
+                    // Set a default for this value - but only if it wasn't specified.
+                    // Allows overriding this in the README where necessary, for testing
+                    // and demonstration.
+                    if !stdout_detection_was_overridden {
+                        cmd.args([
+                            format!("--{stdout_detection_option}"),
+                            "force-tty".into(), // This is prettier for human consumption in the README
+                        ]);
+                    }
+
                     for arg in &inv.args {
                         cmd.arg(arg.to_string());
                     }
@@ -475,7 +499,7 @@ mod tests {
         // anti-pattern? works quite well...)
         let inv = Rc::new(RefCell::new(Invocation::default()));
 
-        let (input, (name, _flags, _args)) = (
+        let (input, (name, _flags, _args, _last_arg)) = (
             maybe_ws(ascii_alpha1),
             many0(alt((
                 map(
@@ -532,7 +556,12 @@ mod tests {
                                     tag("c"),
                                 )),
                                 // Misc. flags used in the docs
-                                alt((tag("glob"), tag("stdin-override-to"), tag("threads"))),
+                                alt((
+                                    tag("stdout-detection"),
+                                    tag("stdin-detection"),
+                                    tag("threads"),
+                                    tag("glob"),
+                                )),
                                 // Shorthands
                                 alt((
                                     tag("tsx"),
@@ -622,6 +651,15 @@ mod tests {
                     },
                 ),
             ))),
+            opt(map(
+                // Handle -- separator followed by the last argument (replacement)
+                preceded((space0, tag("--"), space0), parse_quoted),
+                |s: &str| {
+                    inv.borrow_mut().args.push("--".into());
+                    inv.borrow_mut().args.push(s.into());
+                    s.to_owned()
+                },
+            )),
         )
             .parse(input)?;
 
@@ -848,9 +886,6 @@ mod tests {
                 let mut cmd = Command::try_from(program.clone())
                     .expect("Should be able to convert invocation to cmd to run");
 
-                // We're testing and need determinism. This hard-codes a flag!
-                cmd.arg("--sorted");
-
                 if let Some(previous_stdin) = previous_stdin {
                     cmd.write_stdin(previous_stdin);
                 }
@@ -872,6 +907,9 @@ mod tests {
                     s.to_owned()
                 };
 
+                let observed_stderr = String::from_utf8(assertion.get_output().stderr.clone())
+                    .expect("Stderr should be given as UTF-8");
+
                 if let Some(expected_stdout) = program.stdout().clone() {
                     if cfg!(target_os = "windows") {
                         observed_stdout = fix_windows_output(observed_stdout);
@@ -890,13 +928,15 @@ mod tests {
                             std::fs::write(PATH, new_readme)?;
                         } else {
                             // Write to files for easier inspection
-                            let (mut obs_f, mut exp_f) = (
+                            let (mut obs_stdout_f, mut obs_stderr_f, mut exp_stdout_f) = (
+                                ManuallyDrop::new(NamedTempFile::new().unwrap()),
                                 ManuallyDrop::new(NamedTempFile::new().unwrap()),
                                 ManuallyDrop::new(NamedTempFile::new().unwrap()),
                             );
 
-                            obs_f.write_all(observed_stdout.as_bytes()).unwrap();
-                            exp_f.write_all(expected_stdout.as_bytes()).unwrap();
+                            obs_stdout_f.write_all(observed_stdout.as_bytes()).unwrap();
+                            obs_stderr_f.write_all(observed_stderr.as_bytes()).unwrap();
+                            exp_stdout_f.write_all(expected_stdout.as_bytes()).unwrap();
 
                             // Now panic as usual, for the usual output
                             assert_eq!(
@@ -905,9 +945,10 @@ mod tests {
                                 // whitespace is very hard.
                                 expected_stdout,
                                 observed_stdout,
-                                "Output differs; for inspection see observed stdout at '{}', expected stdout at '{}'",
-                                obs_f.path().display(),
-                                exp_f.path().display()
+                                "Output differs; for inspection see observed stdout at '{}', observed stderr at '{}', expected stdout at '{}'",
+                                obs_stdout_f.path().display(),
+                                obs_stderr_f.path().display(),
+                                exp_stdout_f.path().display()
                             );
 
                             // Temporary files remain, they're not dropped.
@@ -950,10 +991,12 @@ mod tests {
         // Matches: test\some\more\files
         // Matches: test
         // Matches: test\some\file.py
+        // Matches: docs\samples\birds:19:33-36
+        // Matches: docs\samples\birds.py:16:13-16;17-20:
         //```
         //
-        // https://regex101.com/r/NURPe2/1
-        let pattern = Regex::new(r"^([a-z]+\\)*[a-z]+(\.[a-z]+)?$").unwrap();
+        // https://regex101.com/r/NURPe2/2
+        let pattern = Regex::new(r"^([a-z]+\\)+[a-z]+(\.[a-z]+)?").unwrap();
         let mut res = input
             .lines()
             .map(|s| {
